import java.util.*;
import javax.persistence.*;
import java.lang.reflect.*;

import com.sun.ejte.ccl.reporter.SimpleReporterAdapter;

public class TestClient {
	
    EntityManagerFactory emf = 
       Persistence.createEntityManagerFactory("manager", new HashMap());
    EntityManager manager = emf.createEntityManager();
    static boolean dbgFlag = false;

    public TestClient() {
    }

    public boolean create1() {
        
        boolean pass=false;

	PersonPK pkp1 = new PersonPK("pk_p1");
	NaturalPerson p1 = new NaturalPerson();
	p1.setPersonPK(pkp1);
	p1.setName("person1");
	p1.setExt1("ext_p1");

	PersonPK pkp2 = new PersonPK("pk_p2");
	NaturalPerson p2 = new NaturalPerson();
	p2.setPersonPK(pkp2);
	p2.setName("person2");
	p2.setExt1("ext_p2");

	PersonPK pkp3 = new PersonPK("pk_p3");
	NaturalPerson p3 = new NaturalPerson();
	p3.setPersonPK(pkp3);
	p3.setName("person3");
	p3.setExt1("ext_p3");

        // p1 is the parent of p2 and p3
	Vector<NaturalPerson> c = new Vector<NaturalPerson>();
	c.add(p2);
	c.add(p3);
	p1.setChildren(c);

        try {

	    EntityTransaction tx = manager.getTransaction();
            tx.begin();
            manager.persist(p1);
            tx.commit();
            pass = true;
            
        } catch(Throwable e){
            e.printStackTrace();
        }
        
        return pass;
    }

    public boolean find1() {
        
        boolean pass=false;
        String pId = "pk_p1";
        Person person=null; 

        person = findPerson(pId);
        if (person != null && numChildren(person) == 2) {
             pass = true;            
        }
        return pass;
        
    }

    public boolean find2() {
        
        boolean pass=false;
        String pId = "pk_p2";
        Person person=null; 

        person = findPerson(pId);
        if (person != null && numChildren(person) == 0 ) {
             pass = true;            
        }
        return pass;
        
    }

    public Person findPerson(String pId) {
        
        Person person=null; 
        try {
            person = manager.find(Person.class, new PersonPK(pId));
            if (person != null ) {
                dbg(((NaturalPerson)person).toString(), dbgFlag);
                dbg("number of Children="+numChildren(person), dbgFlag);
            } else {
                dbg("No such person", dbgFlag);
	    }
        } catch(Throwable e){
            e.printStackTrace();
        }

        return person;
        
    }

   
    public int numChildren(Person person){
        int count = 0;
        Collection<NaturalPerson> c = ((NaturalPerson)person).getChildren();
        NaturalPerson np= null;
        if (c != null) {
	  Iterator it = c.iterator(); 
	  while (it.hasNext()){
            np = (NaturalPerson) it.next();
            dbg("c:"+np.toString(), dbgFlag);
            count++;
	  }
	}
        return count;
    }

    public boolean executeTest(String testname, Object test) {
	Method m=null;
	boolean ret=false;  
	
	try {
	  m = TestClient.class.getMethod(testname, new Class[] {});
	  Object[] arg = {};
	  ret = (Boolean) m.invoke(test, (Object[])null);     
	} catch (Exception ex){
	  ex.printStackTrace();
	}

	return ret;

     } 
    

    public void doClose() {
	   
	manager.close();
        emf.close();

    }

    public static void dbg(String msg, boolean flag) {
	if (flag) {
            System.out.println(msg);
        }

    }

    public static void main(String[] args) {    

	SimpleReporterAdapter stat = new SimpleReporterAdapter("appserver-sqe");   
	stat.addDescription("EJB3 Issue 1153 Tests");	   
        String testSuiteID = "test";
        String flag = "dbg";

       	String testlist[]={"create1",
                           "find1",
                           "find2"};  

        if (args.length == 1) {
           testSuiteID=args[0];
        } 
        if (args.length == 2) {
           testSuiteID=args[0];
           flag=args[1];
           if (flag.equals(new String("true"))) {
             dbgFlag = true;
	   }
        } 

        TestClient tc = new TestClient();

	for (int i=0; i < testlist.length; i++){
	  String testname=testlist[i];
          dbg("...invoking test" + (i+1) + ": " + testname, dbgFlag);
	  boolean result = tc.executeTest(testname, tc);
	  if (result == true)  {
	    stat.addStatus(testSuiteID + testname, stat.PASS);
	  } else {
	    stat.addStatus(testSuiteID + testname, stat.FAIL);
	  }
          dbg("...finished "+testname+", passed = "+result, dbgFlag);
	}
		
	stat.printSummary(); 
	tc.doClose();

    }
    
}
